@@ -1,14 +1,19 @@
 // use crate::message::BaseMessage;
 
+use crate::tokio::{runtime::Runtime, sync::mpsc::Sender};
 use crate::{
     router::{Router, SenderPair},
     NodeMessage,
 };
-use ockam_core::{Address, Result};
-
-use crate::tokio::{runtime::Runtime, sync::mpsc::Sender};
 use core::future::Future;
 use ockam_core::compat::sync::Arc;
+use ockam_core::{Address, Result};
+
+#[cfg(feature = "std")]
+use ockam_core::{
+    errcode::{Kind, Origin},
+    Error,
+};
 
 /// Underlying Ockam node executor
 ///
@@ -71,7 +76,7 @@ impl Executor {
         crate::block_future(&rt, async move { self.router.run().await })?;
 
         let res = crate::block_future(&rt, async move { join_body.await })
-            .map_err(|_| crate::error::Error::ExecutorBodyJoinError)?;
+            .map_err(|e| Error::new(Origin::Executor, Kind::Unknown, e))?;
 
         Ok(res)
     }
