@@ -13,19 +13,20 @@ pub(super) async fn exec(
 ) -> Result<()> {
     trace!("Stopping worker '{}'", addr);
 
-    let primary_address = if let Some(p) = router.map.addr_map.get(addr) {
-        p.clone()
-    } else {
-        reply
-            .send(RouterReply::no_such_address(addr.clone()))
-            .await
-            .map_err(|_| NodeError::NodeState(NodeReason::Unknown).internal())?;
+    let primary_address = match router.map.addr_map.get(addr) {
+        Some(p) => p.clone(),
+        None => {
+            reply
+                .send(RouterReply::no_such_address(addr.clone()))
+                .await
+                .map_err(|_| NodeError::NodeState(NodeReason::Unknown).internal())?;
 
-        return Ok(());
+            return Ok(());
+        }
     };
 
-    let record = match router.map.internal.remove(&primary_address) {
-        Some(rec) => rec,
+    let record = match router.map.internal.get_mut(&primary_address) {
+        Some(r) => r,
         None => {
             // Actually should not happen
             reply
@@ -46,5 +47,9 @@ pub(super) async fn exec(
         .await
         .map_err(|_| NodeError::NodeState(NodeReason::Unknown).internal())?;
 
+    // Drop worker's Sender to close the worker's mailbox channel
+    // and trigger the worker to start a graceful self-shutdown.
+    record.sender_drop();
+
     Ok(())
 }
